#!/usr/bin/env python
# Copyright (c) 2000-2017 Synology Inc. All rights reserved.

import argparse
import json
import os
import sys
import urllib
import urllib2

class BadParameterException(Exception):
    pass

class ReverseGeocodingRequest:
    '''
    A class to send reverse geocoding request and read the response in
    structural format
    '''
    GET_REQUEST_FORMAT = '{0:s}?{1:s}'
    ENVIRON_KEY_GEOCODING_SERVER_URL = 'SYNO_MOMENTS_GEOCODING_SERVER_URL';
    ENVIRON_KEY_API_KEY = 'SYNO_MOMENTS_API_KEY';
    ENVIRON_KEY_USER_AGENT = 'SYNO_MOMENTS_USER_AGENT';

    def __init__(self):
        self.json_response = {}

    def sendGetRequest(self, para):
        self.getVariable();
        para['syno_serial_number'] = 'TEMP'; # remove after CST removes the check of serial number
        request = urllib2.Request(
            ReverseGeocodingRequest.GET_REQUEST_FORMAT.format(
                self.server_url,
                urllib.urlencode(para)))
        request.add_header('User-Agent', self.user_agent)
        request.add_header('Authorization', 'Key ' + self.api_key)
        request.add_header('Accept', 'application/json')
        request.add_header('Accept-Charset', 'UTF-8')
        return urllib2.urlopen(request, cafile="/etc/ssl/certs/ca-certificates.crt")

    def getVariable(self):
        try:
            self.server_url = os.environ.pop(ReverseGeocodingRequest.ENVIRON_KEY_GEOCODING_SERVER_URL);
            self.api_key = os.environ.pop(ReverseGeocodingRequest.ENVIRON_KEY_API_KEY);
            self.user_agent = os.environ.pop(ReverseGeocodingRequest.ENVIRON_KEY_USER_AGENT);
        except KeyError:
            raise BadParameterException('bad parameter');
        if not self.server_url or not self.api_key or not self.user_agent:
            raise BadParameterException('illegal parameter');

    def send(self, lat, lon):
        para = {
            'lat': lat,
            'lon': lon
        }
        response = self.sendGetRequest(para)
        self.json_response = json.load(response)
        return 0

    def json(self):
        return self.json_response


def parseArgument():
    parser = argparse.ArgumentParser(
        description='A tool to reverse geocoding to human-readable address')
    parser.add_argument(
        '-lat',
        '--latitude',
        type=float,
        dest='lat',
        required=True,
        help='the latitude of the coordinates')
    parser.add_argument(
        '-lon',
        '--longitude',
        type=float,
        dest='lon',
        required=True,
        help='the longitude of the coordinates')
    return parser.parse_args()


def main():
    args = parseArgument()
    request = ReverseGeocodingRequest()
    try:
        request.send(args.lat, args.lon)
    except urllib2.HTTPError as e:
        sys.stderr.write('The server couldn\'t fulfill the request.\n')
        sys.stderr.write('Error code: {0}\n'.format(e.code))
        sys.stderr.write('Full response: {0}\n'.format(e.read()))
        exit(1)
    except urllib2.URLError as e:
        sys.stderr.write('We failed to reach a server.\n')
        sys.stderr.write('Reason: {0}\n'.format(e.reason))
        exit(2)
    except ValueError:
        sys.stderr.write('Failed to decode response as JSON\n')
        sys.stderr.write('Response: {0}\n'.format(response))
        exit(3)
    except BadParameterException as e:
        sys.stderr.write('Bad parameter\n')
        sys.stderr.write('Message: {0}\n'.format(e.message))
        exit(5)

    response = request.json()
    if 'error' in response:
        sys.stderr.write('Failed to reverse geocoding\n')
        sys.stderr.write('Response: {0}\n'.format(response['error']))
        exit(4)

    print json.dumps(response, indent=4)
    exit(0)


if __name__ == "__main__":
    main()
